import './annotations.dart';
import './preconditions.dart';
import './pair.dart';

class Maps {
  static bool isNullOrEmpty<K, V>(@nullable Map<K, V> map) {
    return map == null || map.isEmpty;
  }

  static Map<K, V> fromPairs<K, V>(@nonNull Iterable<Pair<K, V>> iterable) {
    checkNotNull(iterable);
    return Map.fromEntries(iterable.map((t) => MapEntry(t.first, t.second)));
  }
}

extension MapX<K, V> on Map<K, V> {
  /// Returns the value for the given key. If the key is not found in the map,
  /// calls the [defaultValue] function,
  /// puts its result into the map under the given key and returns it.
  ///
  /// Note that the operation is not guaranteed to be atomic
  /// if the map is being modified concurrently.
  @nonNull
  V getOrPut(K key, V defaultValue()) {
    var value = this[key];
    if (value == null) {
      var answer = defaultValue();
      this[key] = answer;
      return answer;
    } else {
      return value;
    }
  }
}
