import 'package:fluartx/fluartx.dart';

class NotImplementedError extends Error {
  final Object message;

  NotImplementedError([this.message = "An operation is not implemented."]);

  @override
  String toString() {
    return message.toString();
  }
}

/// Always throws [NotImplementedError] stating that operation is not implemented.
/// [reason] is a string explaining why the implementation is missing.
TODO(String reason) =>
    throw NotImplementedError("An operation is not implemented: $reason");

R run<R>(R block()) => block();

R let<T, R>(T receiver, R block(T t)) {
  return block(receiver);
}

/// Executes the given function [action] specified number of [times]
repeat(int times, action(int index)) {
  for (int i = 0; i < times; i++) {
    action(i);
  }
}

Function(T t) printFunction<T>() {
  return (t) {
    print(t);
  };
}

extension AnyX on dynamic {
  /// Returns `this` value if it satisfies the given [predicate] or `null`, if it doesn't.
  @nullable
  T takeIf<T>(bool predicate(T t)) {
    if (predicate(this)) {
      return this;
    } else {
      return null;
    }
  }

  /// Returns `this` value if it _does not_ satisfy the given [predicate] or `null`, if it does.
  @nullable
  T takeUnless<T>(bool predicate(T t)) {
    if (!predicate(this)) {
      return this;
    } else {
      return null;
    }
  }

  /// Calls the specified function [block] with `this` value as its argument and returns `this` value.
  T also<T>(Function(T t) block) {
    block(this);
    return this;
  }

  /// Calls the specified function [block] with `this` value as its argument and returns its result.
  R let<T, R>(R block(T t)) {
    return block(this);
  }
}
